/*
   LibIntelCounters -- Support for reading Intel processor Counter
   Copyright (C) 2010 Leonardo de Paula Rosa Piga

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General
   Public License along with this library; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include "libIntelCounters.h"
#include <sys/time.h>

void doSomething() {
  int i, k = 0;

  for (i = 0; i < 1000000; i++)
    k = k + 2;

  sleep(2);
}

int main(int argc, char* argv[]) {
  struct InstCounterSettings iSet0;
  struct InstCounterSettings iSet1;
  struct FixedCounterSettings fSet;
  
  unsigned long long nUInstr, nUCycles;
  unsigned long long nSInstr, nSCycles;
  unsigned long long time;
  struct timeval start, end;
  // Setting up structures
  // Count user instructions
  iSet0.counter = 0;
  iSet0.user = 1;
  iSet0.sys = 0;
  // Count kernel instructions
  iSet1.counter = 1;
  iSet1.user = 0;
  iSet1.sys = 1;
  // Count user cycles
  fSet.user = 1;
  fSet.sys = 0;

  // Reset counters
  resetInstructionCounter(&iSet0);
  resetInstructionCounter(&iSet1);
  resetCpuCoreCyclesCounter();

  gettimeofday(&start,NULL);
  // Enable counters
  enableInstructionCounter(&iSet0);
  enableInstructionCounter(&iSet1);
  enableCpuCoreCyclesCounter(&fSet);

  // Do some process
  doSomething();

  // Disable counters
  disableInstructionCounter(&iSet0);
  disableInstructionCounter(&iSet1);
  disableCpuCoreCyclesCounter();
  gettimeofday(&end,NULL);

  time = start.tv_sec * 1000000 + start.tv_usec;
  time = (end.tv_sec * 1000000 + end.tv_usec) - time;

  // Read number of instructions
  nUInstr = instructionCounter(&iSet0);

  nSInstr = instructionCounter(&iSet1);

  nUCycles = cpuCoreCyclesCounter();


  // Repeat, but now counting system cycles

  // Count system cycles
  fSet.user = 0;
  fSet.sys = 1;
  resetCpuCoreCyclesCounter();
  enableCpuCoreCyclesCounter(&fSet);

  // Do some process
  doSomething();

  disableCpuCoreCyclesCounter();
  nSCycles = cpuCoreCyclesCounter();

  // Display the results
  printf("Instructions:\n");
  printf("User Instructions = %Ld\n", nUInstr);
  printf("Sys Instructions = %Ld\n", nSInstr);
  printf("Cycles:\n");
  printf("Sys Cycles = %Ld\n", nSCycles);
  printf("User Cycles = %Ld\n", nUCycles);
  printf("Total time = %.6f\n", (time / 1000000.));
  // Printing some metrics
  double ipc;
  ipc = ((double) (nSInstr + nUInstr)) / ((double) (nUCycles + nSCycles));
  printf("\nIPC =%.3f\n", ipc);

  ipc = ((double) nUInstr) / ((double) (nUCycles));
  printf("User IPC =%.3f\n", ipc);
  
  printf("\nFrequency = %.3f\n", (nUCycles + nSCycles) / (time / 1000000.));

  return 0;
}
